{% extends "ksp_login/base.html" %}

{% load i18n %}

{% block ksp_login_title %}{% trans "Password change" %}{% endblock ksp_login_title %}

{% block ksp_login_content %}
    {% include "ksp_login/parts/single_form.html" %}
{% endblock ksp_login_content %}
